@@ -16,10 +16,12 @@ class Spawn(objects.Object):
 		command="ls",
 		# asynchronous.
 		async_=False,
-		# system options.
-		response_str=None,
+		# the log level.
+		log_level=0,
 		# additional attributes.
 		attributes={},
+		# system options.
+		response_str=None,
 	):
 
 		# defaults.
@@ -29,6 +31,7 @@ class Spawn(objects.Object):
 		# args.
 		self.command = command
 		self.async_ = async_
+		self.log_level = log_level
 		self.response_str = response_str
 
 		# vars.
@@ -87,16 +90,16 @@ class Spawn(objects.Object):
 				if not isinstance(r, (int, Integer, float)):
 					return r3sponse.error(f"Expected [{expect}] is not the child's expected input (#873465).")
 				c = 2
-				if defaults.options.log_level >= 8: print(f"Found expected: {expect}")
+				if self.log_level >= 8: print(f"Found expected: {expect}")
 				if send != None:
-					if defaults.options.log_level >= 8: print(f"Attempting to send: {send}")
+					if self.log_level >= 8: print(f"Attempting to send: {send}")
 					#if self.async_:
 					#	g = yield from self.child.sendline(str(send), async_=True)
 					#	r = next(g)
 					#else:
 					r = self.child.sendline(str(send))
 					self.__secrets__.append(str(send))
-					if defaults.options.log_level >= 8: print(f"Succesfully sended: {send}")
+					if self.log_level >= 8: print(f"Succesfully sended: {send}")
 			except pexpect.exceptions.TIMEOUT:
 				if c == 1:
 					return r3sponse.error(f"Expected [{expect}] is not the child's expected input, error: Timeout (during epxecting).")
@@ -135,7 +138,7 @@ class Spawn(objects.Object):
 						raise exceptions.InvalidUsage(f"{self.__traceback__(function='expect', parameter='send')}: parameter [send] and parameter [expect] do not have the same indexes.")
 				else:
 					_send_ = str(send)
-				if defaults.options.log_level >= 8: print(f"Checking optional expect: {_expect_}")
+				if self.log_level >= 8: print(f"Checking optional expect: {_expect_}")
 				response = self.expect(expect=_expect_, timeout=timeout, send=_send_)
 				if not response.success and "is not the child's expected input" not in response.error: return response
 				elif response.success:
@@ -188,7 +191,7 @@ class Spawn(objects.Object):
 					break
 				except Exception as e:
 					if timeout != None: self.child.timeout = old
-					if defaults.options.log_level <= 0: e = str(e).split("\n")[0] # strip the full child from the pexpect error message.
+					if self.log_level <= 0: e = str(e).split("\n")[0] # strip the full child from the pexpect error message.
 					while True:
 						if len(e) >= 1 and e[len(e)-1] == ".": e = e[:-1]
 						else: break
@@ -213,7 +216,7 @@ class Spawn(objects.Object):
 				for i in lines: output += i.decode()
 			except Exception as e:
 				self.child.timeout = old
-				if defaults.options.log_level <= 0: e = str(e).split("\n")[0] # strip the full child from the pexpect error message.
+				if self.log_level <= 0: e = str(e).split("\n")[0] # strip the full child from the pexpect error message.
 				while True:
 					if len(e) >= 1 and e[len(e)-1] == ".": e = e[:-1]
 					else: break
@@ -486,7 +489,7 @@ def execute(
 	arguments=[], 
 	#
 	# Options:
-	#   asynchronous (cant capture output).
+	#   asynchronous process.
 	async_=False,
 	#	await asynchronous child (sync process always awaits).
 	wait=False,
@@ -503,10 +506,14 @@ def execute(
 	#	  dict instance: selects "and" mode ; send expected inputs and their value & return error when one of them is missing.
 	#	  list[dict] instance: send all dictionaries in the list (default dict behaviour so one of the keys in each dict is expected).
 	input=None,
-	#   do not throw an error when the input is missing or not expected when optional is disabled (bool).
-	optional=False, 
 	#   the input timeout (float) (list with floats by index from input)
 	timeout=1.0,
+	#   do not throw an error when the input is missing or not expected when optional is disabled (bool).
+	optional=False, 
+	#
+	# Logging.
+	#   the log level.
+	log_level=defaults.options.log_level,
 	#
 	# System functions.
 	#   add additional attributes to the spawn object.
@@ -540,11 +547,12 @@ def execute(
 	l = []
 	for i in arguments: l.append(f'"{i}"')
 	arguments = Array(l).string(joiner=' ')
-	if defaults.options.log_level >= 8: print(f"Spawn: [ $ {executive} {path} {arguments}]",)
+	if log_level >= 8: print(f"Spawn: [ $ {executive} {path} {arguments}]",)
 	spawn = Spawn(
 		command=f"{executive} {path} {arguments}",
 		#async_=async_, # does not work.
 		response_str=response_str,
+		log_level=log_level,
 		attributes=__spawn_attributes__,
 	)
 	spawn.echo = False
@@ -601,7 +609,7 @@ def execute(
 				send = list(dictionary.values())
 
 				# expect .
-				if defaults.options.log_level >= 8:
+				if log_level >= 8:
 					print(f"Expecting one of the following inputs: {expect}.")
 				response = spawn.expect(expect=expect, send=send, timeout=timeout)
 				if not response.success:
@@ -611,51 +619,57 @@ def execute(
 						else:
 							return r3sponse.error(f"Specified input [{Array(expect).string(joiner=', ')}] was not expected.")
 					else: return response
-				if defaults.options.log_level >= 8:
+				if log_level >= 8:
 					print("Send response message:",response.message)
 
 				# success.
 				return r3sponse.success("Success.")
 
 			""" check expecting. """
+			expecting = True
 			if not spawn.expecting:
-				if defaults.options.log_level >= 1:
-					return OutputObject(error=f"Failed to send {str_input} to {response_str}, child is not expecting any input [{spawn.child}].")
-				else:
-					return OutputObject(error=f"Failed to send {str_input} to {response_str}, child is not expecting any input.")
+				expecting = False
+				if not optional:
+					if log_level >= 1:
+						return OutputObject(error=f"Failed to send expected input {str_input} to {response_str}, child is not expecting any input [{spawn.child}].")
+					else:
+						return OutputObject(error=f"Failed to send expected input {str_input} to {response_str}, child is not expecting any input.")
 
-			# send all dicts in the list (list instance).
-			error_end_of_file = False
-			if isinstance(input, (list, Array)):
-				for _input_ in input:
-					response = process_dict_input(_input_)
+			# limit not expecting by optional.
+			if expecting:
+
+				# send all dicts in the list (list instance).
+				error_end_of_file = False
+				if isinstance(input, (list, Array)):
+					for _input_ in input:
+						response = process_dict_input(_input_)
+						if not response.success: 
+							if "End of file" in response.error:
+								error_end_of_file = True
+							else: 
+								# str input.
+								if isinstance(_input_, (list, Array)):
+									str_input = Array(_input_).string(joiner=", ")
+								elif isinstance(_input_, (dict, Dictionary)):
+									str_input = Array(list(_input_.keys())).string(joiner=", ")
+								else: 
+									str_input = f"[{_input_}]"
+								return OutputObject(error=f"Failed to send one of the expected input(s) {str_input}, error: {response.error}")
+
+				# send one of the keys (dict instance).
+				elif isinstance(input, (dict, Dictionary)):
+					response = process_dict_input(input)
 					if not response.success: 
 						if "End of file" in response.error:
 							error_end_of_file = True
-						else: 
-							# str input.
-							if isinstance(_input_, (list, Array)):
-								str_input = Array(_input_).string(joiner=", ")
-							elif isinstance(_input_, (dict, Dictionary)):
-								str_input = Array(list(_input_.keys())).string(joiner=", ")
-							else: 
-								str_input = f"[{_input_}]"
-							return OutputObject(error=f"Failed to send one of the expected input(s) [{str_input}], error: {response.error}")
+						else: return OutputObject(error=f"Failed to send one of the expected input(s) {str_input}, error: {response.error}")
 
-			# send one of the keys (dict instance).
-			elif isinstance(input, (dict, Dictionary)):
-				response = process_dict_input(input)
-				if not response.success: 
-					if "End of file" in response.error:
-						error_end_of_file = True
-					else: return OutputObject(error=f"Failed to send one of the expected input(s) [{str_input}], error: {response.error}")
-
-			""" check no input left (does not work properly).
-			if not error_end_of_file and spawn.expecting:
-				try: after = spawn.child.after.decode()
-				except : after = spawn.child.after
-				return OutputObject(error=f"Failed to execute {response_str}, still expecting: [{after}].")
-			"""
+				""" check no input left (does not work properly).
+				if not error_end_of_file and spawn.expecting:
+					try: after = spawn.child.after.decode()
+					except : after = spawn.child.after
+					return OutputObject(error=f"Failed to execute {response_str}, still expecting: [{after}].")
+				"""
 			
 
 		# do not get or kill when async.
@@ -671,14 +685,14 @@ def execute(
 			if not response.success: return OutputObject(error=response.error)
 			output = response.output
 			if error_end_of_file != None and error_end_of_file: 
-				if defaults.options.log_level >= 1:
-					return OutputObject(error=f"Failed to send the expected input to {response_str} (#234343) (output: {output}) (child: {spawn.child}).")
+				if log_level >= 1:
+					return OutputObject(error=f"Failed to send expected input {str_input} to {response_str} (#234343) (output: {output}) (child: {spawn.child}).")
 				else:
-					return OutputObject(error=f"Failed to send the expected input to {response_str} (#234343) (output: {output}).")
+					return OutputObject(error=f"Failed to send expected input {str_input} to {response_str} (#234343) (output: {output}).")
 
 			# check kill.
 			if kill and spawn.running:
-				if defaults.options.log_level >= 8: print(f"Killing process {response_str}.")
+				if log_level >= 8: print(f"Killing process {response_str}.")
 				response = spawn.kill()
 				if not response.success: return OutputObject(error=f"Failed to kill {response_str}, error: {response.error}")
 
@@ -705,7 +719,7 @@ def execute(
 				
 				# check kill.
 				if kill and spawn.running:
-					if defaults.options.log_level >= 8: print(f"Killing process {response_str}.")
+					if log_level >= 8: print(f"Killing process {response_str}.")
 					response = spawn.kill()
 					if not response.success: return OutputObject(error=f"Failed to kill {response_str}, error: {response.error}")
 
@@ -757,7 +771,7 @@ def execute(
 			error_, output = error.stderr, error.output
 			if isinstance(error_, bytes): error_ = error_.decode()
 			if isinstance(output, bytes): output = output.decode()
-			if defaults.options.log_level <= 0:
+			if log_level <= 0:
 				return OutputObject(error=f"Failed to execute {response_str}, (exception: {exception}), (error: {error_}).")
 			else:
 				return OutputObject(error=f"Failed to execute {response_str}, (exception: {exception}), (output: {output}), (error: {error_}).")
@@ -765,7 +779,7 @@ def execute(
 		if isinstance(error_, bytes): error_ = error_.decode()
 		if isinstance(output, bytes): output = output.decode()
 		if error_ != "":
-			if defaults.options.log_level <= 0:
+			if log_level <= 0:
 				return OutputObject(error=f"Failed to execute {response_str}, (error: {error_}).")
 			else:
 				return OutputObject(error=f"Failed to execute {response_str}, (output: {output}), (error: {error_}).")
